/*
    libut61 - library for parsing UT61 (ES51922) DMM readout data stream
    Copyright (C) 2021  Tibor 'Igor2' Palinkas

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

    Project page:           http://www.repo.hu/projects/ut61
    Version control:        svn://svn.repo.hu/ut61/trunk
    Contact (email & IRC):  http://igor2.repo.hu/contact.html
*/

#define _DEFAULT_SOURCE
#include <stdlib.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <termios.h>
#include <unistd.h>

#include "ut61.h"
#include "debug.h"

int ut61_verbose = 0;

int ut61_port_open(const char *fn)
{
	int fd, RTS_flag = TIOCM_RTS, DTR_flag = TIOCM_DTR;
	struct termios tios;

	tprintf(3, "Opening port %s...\n", fn);
	fd = open(fn, O_RDWR | O_NOCTTY);
	if (fd == -1) {
		perror("ut61: open() - can not open the port for read/write");
		exit(1);
	}

	tprintf(3, "Configuring port %s...\n", fn);
	if (tcgetattr(fd, &tios) != 0) {
		perror("ut61: tcgetattr() - can not read serial settings ");
		exit(1);
	}

	tios.c_cflag &= ~CBAUD;
	tios.c_cflag |= B19200;

	tios.c_cflag &= ~CSIZE;
	tios.c_cflag |= CS7;

	tios.c_cflag &= ~CSTOPB;
	tios.c_cflag |= PARENB | PARODD;

	if (tcsetattr(fd, TCSANOW, &tios) != 0) {
		perror("ut61: tcsetattr() - can not read serial settings ");
		exit(1);
	}

	/* Provide power */
	ioctl(fd, TIOCMBIC, &RTS_flag); /* Clear RTS pin */
	ioctl(fd, TIOCMBIS, &DTR_flag); /* Set DTR pin */

	tprintf(1, "Port %s opened\n", fn);

	return fd;
}

/* Read till the first newline to start with a clean state */
void ut61_port_sync(int fd)
{
	int got = 0;

	tprintf(3, "Sync: waiting for the first separator...\n");

	for(got = 0; got < 16; got++) {
		char c;
		read(fd, &c, 1);
		if (c == '\n')
			return;
	}

	fprintf(stderr, "Sync failed: Failed to read separator while receiving noise\n");
	exit(1);
}

#define get_char() \
	do { \
		if (read(fd, &c, 1) != 1) {\
			pkt->synced = 0; \
			return -1; \
		} \
		tprintf(4, "Read: %02x\n", c); \
	} while(0)

#define req011(c) \
	do { \
		if ((c & 0xF0) != 0x30) { \
			tprintf(2, "Syntax error: high nibble must be 3\n"); \
			pkt->synced = 0; \
			return -1; \
		} \
	} while(0)

int ut61_packet_read(int fd, ut61_packet_t *pkt)
{
	unsigned char c;
	int n;

	pkt->valid = 0;
	pkt->raw_valid = 0;

	if (!pkt->synced) {
		ut61_port_sync(fd);
		pkt->synced = 1;
	}

	/* #0: range */
	do {
		get_char();
	} while((c == '\r') || (c == '\n'));
	pkt->range = c;

	/* #1..#5 digits from 4 to 0 */
	for(n = 4; n >= 0; n--) {
		get_char();
		pkt->dig[n] = c;
	}

	/* #6: function */
	get_char();
	pkt->func = c;

	/* #7: status */
	get_char();
	req011(c);
	pkt->judge = !!(c & 8);
	pkt->sign = !!(c & 4);
	pkt->batt = !!(c & 2);
	pkt->ol = !!(c & 1);

	/* #8: option 1 */
	get_char();
	req011(c);
	pkt->max = !!(c & 8);
	pkt->min = !!(c & 4);
	pkt->rel = !!(c & 2);
	pkt->rmr = !!(c & 1);

	/* #9: option 2 */
	get_char();
	req011(c);
	pkt->ul = !!(c & 8);
	pkt->pmax = !!(c & 4);
	pkt->pmin = !!(c & 2);

	/* #10: option 3 */
	get_char();
	req011(c);
	pkt->dc = !!(c & 8);
	pkt->ac = !!(c & 4);
	pkt->automatic = !!(c & 2);
	pkt->vahz = !!(c & 1);


	/* #11: option 4 */
	get_char();
	req011(c);
	pkt->vbar = !!(c & 4);
	pkt->hold = !!(c & 2);
	pkt->lpf = !!(c & 1);

	/* #12 and #13 are terminators */
	get_char();
	if ((c != '\r') && (c != '\n')) {
		tprintf(1, "Synatx error: expected CR, got %02x\n", c);
		return -1;
	}

	get_char();
	if ((c != '\r') && (c != '\n')) {
		tprintf(1, "Synatx error: expected LF, got %02x\n", c);
		pkt->synced = 0;
		return -1;
	}

	pkt->raw_valid = 1;
	return 0;
}

void ut61_packet_fprint(FILE *f, const ut61_packet_t *pkt)
{
	fprintf(f, "# [%c%c%c%c%c]\n", pkt->dig[4], pkt->dig[3], pkt->dig[2], pkt->dig[1], pkt->dig[0]);

	fprintf(f, "#  %s battery=%s judge=%d vbar=%d sign=%d hold=%d lpf=%d\n",
		pkt->raw_valid ? "valid" : "INVALID", pkt->batt ? "LOW" : "ok",
		pkt->judge, pkt->vbar, pkt->sign, pkt->hold, pkt->lpf);

	fprintf(f, "#  dc=%d ac=%d auto=%d OL=%d min=%d max=%d rel=%d\n",
		pkt->dc, pkt->ac, pkt->automatic, pkt->ol, pkt->min, pkt->max, pkt->rel);

	fprintf(f, "#  rmr=%d ul=%d pmax=%d pmin=%d vahz=%d func=%02x range=%02x\n",
		pkt->rmr, pkt->ul, pkt->pmax, pkt->pmin, pkt->vahz, pkt->func, pkt->range);
}
