/*
    libut61 - library for parsing UT61 (ES51922) DMM readout data stream
    Copyright (C) 2021  Tibor 'Igor2' Palinkas

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

    Project page:           http://www.repo.hu/projects/ut61
    Version control:        svn://svn.repo.hu/ut61/trunk
    Contact (email & IRC):  http://igor2.repo.hu/contact.html
*/

#define _DEFAULT_SOURCE
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>

#include <libut61/ut61.h>
#include <libut61/digest.h>
#include <libut61/raw.h>

static const char *def_template = "$(info)    $(vbas) $(ubas)    $(val) $(unit)\n";

static void help_default(void)
{
	printf("# Default template:\n%s\n", def_template);
}

static void help_template(void)
{
#include "help_temp.h"
}

static void help_options(void)
{
#include "help_cli.h"
}

static void help_generic(const char *prog, const char *topic)
{
	printf("ut61 - parse the serial stream from an UT61 DMM, print values on stdout\n");
	printf("Usage: %s [options] [-f template] /dev/portfile\n\n", prog);
	printf("For more info, try --help topic\n");
	printf("Topics:\n");
	printf("  options        description of options\n");
	printf("  template       list all $(vars) in the template language\n");
	printf("  default        default template string\n");
	printf("\nVersion: %d.%d.%d\n\n", UT61_VER_MAJOR, UT61_VER_MINOR, UT61_VER_PATCH);
}



static void help(const char *prog, const char *topic)
{
	if ((topic == NULL) || (*topic == '\0')) help_generic(prog, topic);
	else if (strcmp(topic, "default") == 0) help_default();
	else if (strcmp(topic, "template") == 0) help_template();
	else if (strcmp(topic, "options") == 0) help_options();
	else {
		printf("ERROR: Unknown help topic '%s'\n\n", topic);
		help_generic(prog, topic);
	}
}

int main(int argc, char *argv[])
{
	long count;
	int skp = 0, skip = 0, inf = 1, n, fd, debug = 0;
	const char *template = def_template;
	const char *portfn = NULL;
	ut61_packet_t pkt = {0};

	for(n = 1; n < argc; n++) {
		char *cmd = argv[n], *arg = argv[n+1];
		if (*cmd == '-') {
			while(*cmd == '-') cmd++;
			switch(*cmd) {
				case 'd': debug = 1; break;
				case 'v': ut61_verbose++; break;
				case 'f': template = arg; n++; break;
				case 'h': help(argv[0], arg); exit(0);
				case 'c':
					{
						char *end;

						inf = 0;
						count = strtol(arg, &end, 10);
						n++;
						if (*end != '\0') {
							fprintf(stderr, "-c needs an integer count instead of '%s'\n", arg);
							exit(1);
						}
					}
					break;
				case 's':
					{
						char *end;

						skp = skip = strtol(arg, &end, 10);
						n++;
						if (*end != '\0') {
							fprintf(stderr, "-s needs an integer skip instead of '%s'\n", arg);
							exit(1);
						}
					}
					break;
			}
		}
		else {
			if (portfn != NULL) {
				fprintf(stderr, "Multiple ports not supported ('%s' vs. '%s')\n", portfn, argv[n]);
				exit(1);
			}
			portfn = argv[n];
		}
	}

	if (portfn == NULL) {
		fprintf(stderr, "Need a port dev path, e.g. /dev/ttyS1\n");
		exit(1);
	}

	fd = ut61_port_open(portfn);
	if (fd == -1)
		exit(1);

	/* main loop - keep printing data */
	while(inf || (count > 0)) {
		ut61_packet_read(fd, &pkt);

		if ((skip != 0) && (skp < skip)) {
			skp++;
			continue;
		}
		skp = 0;

		ut61_digest(&pkt);

		if (debug) {
			ut61_packet_fprint(stdout, &pkt);
			printf("# {%f %s} valid=%d {%f %s} in %s; LCD: %s %s\n", pkt.value, pkt.unit, pkt.valid, pkt.base_value, pkt.base_unit, pkt.func_name, pkt.value_str, pkt.unit);
		}

		ut61_template_fprint(stdout, &pkt, template);
		count--;
	}

	close(fd);
	return 0;
}
